#
# BIND IPV4 classes
#
# Jason Vas Dias <jvdias@redhat.com>, Red Hat Inc. Sep 2004

import DNS

import string
import re

REVERSE=-1
REVERSE_ZONE='IN-ADDR.ARPA.'

class Address(DNS.Data):    
    addr=''
    prefix=None
    implicit_prefix=None
    ip4re=re.compile(r'^([0-9]{1,3})(((\.([0-9]){1,3}){1,3})?)$')
    ip4_prefix_re=re.compile(r'^([0-9]{1,3})(((\.([0-9]){1,3}){1,3})?)((/([0-9]{1,2}))?)$')
    def __init__(self, addr, input=False, prefixAllowed=False):
        if not input:
            self.addr=addr
        else:
            self.errors=self.input(addr,prefixAllowed)            

    def __str__(self):
        if( self.addr != None ):
            if( self.prefix != None) and (self.prefix >= 0):
                return self.addr + '/' + str(self.prefix)            
            return self.addr
        else:
            return ''

    def octs(self):
        if (self.addr == None ) or (len(self.addr)==0):
            return []
        octs=self.addr.split('.')
        return octs

    def hex(self):
        octs=self.octs()
        while len(octs) < 4:
            octs.append('0')
        return( ('%0.2x' % int(octs[0]))
               +('%0.2x' % int(octs[1]))
               +('%0.2x' % int(octs[2]))
               +('%0.2x' % int(octs[3]))
              )

    def int(self):
        return int(self.hex(),16)
    
    def input(self, ip, prefixAllowed=False):
        self.addr=''
        self.prefix=None
        self.implicit_prefix=None
        if not prefixAllowed :
            if not self.ip4re.match(ip):
                return (-1,_("invalid IPv4 address:") + ' ' + ip)
        else:
            prefix = None
            if not self.ip4_prefix_re.match(ip):
                return (-1,_("invalid IPv4 prefix:") + ' ' + ip)
            pfx_i=ip.find('/')
            if( pfx_i > -1 ):
                prefix=int(ip[pfx_i+1:])
                if(prefix > 32):
                    return (-1,_("invalid IPv4 prefix:") + ' ' + ip)
                ip=ip[:pfx_i]
#            else:
#                a=ip.split('.')
#                prefix = len(a) * 8
#                self.implicit_prefix = True
            self.prefix = prefix
        a=ip.split('.')
        for o in a:
            if( (o == None) or (len(o) < 0)):
                return (-1,("invalid null IPV4 octet:") + ' ' + ip)
            i = int(o)
            if (i < 0) or (i > 255):
                return (-1,("invalid IPv4 address octet:") + ' ' + o)
        self.addr = ip
        return None
    
    def check(self, ip, prefixAllowed=False):
        a=Address(ip)
        r = a.input(ip)
        return r    

    def output(self):
        s=''
        octs=self.octs()
        for o in octs:
            s += o + '.'
        s=s[:-1]
        if( self.prefix != None):# and (self.implicit_prefix == None):
            s += '/' + str(self.prefix)
        return s

class AddressOrPrefix(Address):
    def check(self, ip):
        return Address.check(self, ip, True)

def reverse(ip):
    a=ip.split('.')
    s=''
    for i in range(len(a)-1,-1,-1):
        s += a[i] + '.'
    return s[:-1]

class ReverseAddress(DNS.Data):    
    addr=None
    origin=None

#    ip4re=re.compile(r'^([0-9]{1,3})(((\.([0-9]){1,3}){1,3})?)((\.IN-ADDR.ARPA\.)?)$',re.I)
    ip4re=re.compile(r'^([0-9]{1,3})(((\.([0-9]){1,3}){1,3})?)((\.IN-ADDR.ARPA\.)?)$',re.I)
    
    def __init__(self, addr, zone):
        self.addr=addr
        if(len(self.addr)<=0):
            if (zone.zone_origin == None) or (len(zone.zone_origin) <= 0):
                self.addr='0.'
            else:
                self.addr=zone.zone_origin
        self.origin=zone.origin
        self.zone_origin=zone.zone_origin

    def value(self):
        return self.addr
                
    def __str__(self):        
        if (self.addr==None) and (self.origin==None):
            return ''
        elif (self.addr==None) or (self.addr == '@'):
            return self.origin
        elif self.addr[-1] == '.':
            return self.addr
        elif self.origin != '.':
            return self.addr + '.' + self.origin
        else:
            return self.addr + self.origin

    def strip_arpa(self, a=None):
        if(a == None):
            s=self.__str__()
        else:
            s=a
        if s[-len(REVERSE_ZONE):].upper() == REVERSE_ZONE:
            s=s[:-len(REVERSE_ZONE)]
        if (len(s)) and ( s[-1]=='.'):
            s=s[:-1]
        return s

    def reverse(self, addr=None):
        if( addr == None ):
            s = self.strip_arpa()
        else:
            s = self.strip_arpa(addr)
        a=s.split('.')
        l=''
        for i in range(len(a)-1,-1,-1):
            l += a[i]+'.'
        if len(l) and ( l[-1]=='.'):
            l=l[:-1]            
        return l

    def contains(s, ip):
        a = s.reverse(s.zone_origin)
        return ((len(a)>0) and (len(ip)>0) and ( len(ip) > len(a) ) and ( ip[:len(a)].upper() == a.upper() ))
    
    def label(self):
        return self.reverse()

    def relative_label(self):
        s=self.reverse()        
        o=self.reverse(self.zone_origin)
        if len(s) > len(o):
            s = s[-((len(s)-len(o))-1):]
        elif len(s) == len(o):
            s = ''
        return s

    def address(self):
        s=self.reverse()        
        o=self.reverse(self.zone_origin)
        sa=Address(s)
        oa=Address(o)
        oa.input(o,True)
        sa.origin_address = oa
        debug( 'IPV4 Reverse Address:  self: ', sa.addr, 'p:', sa.prefix, 'origin:', oa, 'p:', oa.prefix)
        return sa
    
    def check(self, ip):
        if (ip == None) or (len(ip) == 0):
            return (-1,_('null reverse ip address'))
        if (ip[-1] != '.'):
            if self.origin != '.':
                ip += '.' + self.origin
            else:
                ip += self.origin
        if (ip[-len(self.zone_origin):].upper() != self.zone_origin.upper()):
            return (-1,_("IPv4 address %s is out of zone %s") %(ip, self.origin))
        if not self.ip4re.match(ip):
            return (-1,_('invalid reverse ip address: %s') % ip)
        ip = ip[:-len(self.origin)]
        a = ip.split('.')
        i = 0        
        for o in a:
            i+=1
            if ((o == None) or (len(o) == 0)):
               if (i != len(a)):
                   return (-1, _('invalid reverse ip address: %s') % ip)
               else:
                   continue
            if (int(o) < 0) or (int(o) > 255):
                return (-1,_('invalid octet: %s') % o)
        return None

    def split_domain(s,n=None):
        if n==None:
            n=str(s)
        a=n.split('.')
        return (a[0],DNS.make_absolute(string.join(a[1:],'.')))
    
    def relative_value(self, origin=None):
        if origin == None:
            if has_attr(self,'zone_origin'):
                origin = self.zone_origin
            else:
                origin = ''
        s = self.__str__()
        if (len(origin)>0) and (len(s) >= len(origin)) and (s[-len(origin):]==origin):
            s=s[:-len(origin)]
            if( len(s) == 0):
                s = '@'
            if(len(s) > 1) and (s[-1] == '.'):
                s=s[:-1]
        return s
